/*!
// @file		EmaiMessage.h
//
// @brief		Message functionality, used for composing and parsing MIME messages
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAIMESSAGE_H__
#define __EMAIMESSAGE_H__

#include <EmaiObject.h>

#ifdef __cplusplus
	extern "C" {
#endif

/*!
// @brief	EmaiMessage represents message object in RFC822 format
*/
typedef struct EmaiOpaqueMessage*				EmaiMessage;

/*!
// @brief	EmaiMessagePart represents message part of the RFC822 message
*/
typedef struct EmaiOpaqueMessagePart*			EmaiMessagePart;

/*!
// @brief	EmaiStringList is an object which allow you to hold a list of unicode
//			strings and access them by index
*/
typedef struct EmaiOpaqueStringList*			EmaiStringList;

/*!
// @brief	EmaiContactList is an object which simplify operations with contacts
//			(from, to, cc etc) in message object
*/
typedef struct EmaiOpaqueContactList*			EmaiContactList;

/*!
// @brief	EmaiContentField object can be used to setup or retrieve parsed field data
//				for Content-Type and Content-Disposition fields.
//			For Content-Type field value member contains media type of the message part
//				("text/plain", "application/octet-stream", etc)
//			For Content-Disposition field value member contains disposition of the
//				message part ("attachment", "inline" etc)
*/
typedef struct EmaiOpaqueContentField*			EmaiContentField;


/*!
// @brief	Character sets constants
//
// @see		EmaiPartData
// @see		EmaiPlainTextPartData
*/
enum EmaiEnumMessageCharset
{
	EmaiUSASCII,
	EmaiKOI_8R,
	EmaiKOI_8U,
	EmaiISO8859_1,
	EmaiISO8859_2,
	EmaiISO8859_3,
	EmaiISO8859_4,
	EmaiISO8859_5,
	EmaiISO8859_6,
	EmaiISO8859_7,
	EmaiISO8859_8,
	EmaiISO8859_9,
	EmaiISO8859_10,
	EmaiISO8859_11,
	EmaiISO8859_13,
	EmaiISO8859_14,
	EmaiISO8859_15,
	EmaiISO8859_16,
	EmaiWindows1250,
	EmaiWindows1251,
	EmaiWindows1252,
	EmaiWindows1253,
	EmaiWindows1254,
	EmaiWindows1255,
	EmaiWindows1256,
	EmaiWindows1257,
	EmaiWindows1258,
	EmaiBig5,
	EmaiKSC5601_1987,
	EmaiGB2312,
	EmaiISO_2022_JP,
	EmaiISO_2022_JP_2,
	EmaiISO_2022_JP_1,
	EmaiISO_2022_JP_3,
	EmaiCSISO_2022_JP,
	EmaiEUC_JP,
	EmaiISO_2022_CN,
	EmaiISO_2022_CN_EXT,
	EmaiHZ_GB_2312,
	EmaiISO_2022_KR,
	EmaiEUC_KR,
	EmaiUTF_8,
	EmaiMacRomanLatin1,

	EmaiUnknown							= 0xFFFFFFFF
};


/*!
// @brief	Creates emtpy string list object.
//
// @param	outStringList - on successful exit contains created list
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiStringListCreate(
		EmaiStringList* outStringList,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves the number of the item at the string list.
//
// @param	inStringList - string list to be examined
// @param	outCount - on successful exit contains number of items in the list
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiStringListGetCount(
		EmaiStringList inStringList,
		EmaiUint32* outCount,
		EmaiOptions inOptions);

/*!
// @brief	Adds the string to the end of the string list.
//
// @param	inStringList - string list to be modified
// @param	inString - the string to be added to the list
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiStringListAddString(
		EmaiStringList inStringList,
		EmaiConstUniCharPtr inString,
		EmaiOptions inOptions);

/*!
// @brief	Extracts friendly contact name from the contact list, at the specified index.
//
// @param	inStringList - string list to be examined
// @param	inIndex - zero-based index of the contact at the list
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Zero-terminated unicode string, which contains the requested string.
//				The function returns NULL if invalid parameters are used
//				(i.e. inStringList is not valid list, or inIndex is out of range)
*/
EMAIAPI(EmaiConstUniCharPtr)
EmaiStringListGetString(
		EmaiStringList inStringList,
		EmaiUint32 inIndex,
		EmaiOptions inOptions);


/*!
// @brief	Creates an emtpy contact list object.
//
// @param	outContactList - on successful exit contains created list
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiContactListCreate(
		EmaiContactList* outContactList,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves the number of records in contact list.
//
// @param	inContactList - contact list to be examined
// @param	outCount - on successful exit contains number of the records in the list
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
// @see		Emai::ContactList::GetCount
*/
EMAIAPI(EmaiError)
EmaiContactListGetCount(
		EmaiContactList inContactList,
		EmaiUint32* outCount,
		EmaiOptions inOptions);

/*!
// @brief	Adds a contact to the contact list.
//
// @param	inContactList - contact list to be modified
// @param	inName - friendly contact name.\n
//				The parameter inName can also be used to pass a
//				combined value (name and address) in the format
//				of "name" <address> to the plug-in. In this case
//				the inAddress parameter should be empty.
// @param	inAddress - e-mail address of the contact
// @param	inReserved - Reserved for future version, pass NULL
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
// @see		Emai::ContactList::AddContact
*/
EMAIAPI(EmaiError)
EmaiContactListAddContact(
		EmaiContactList inContactList,
		EmaiConstUniCharPtr inName,
		EmaiConstUniCharPtr inAddress,
		const void* inReserved, /*must be NULL*/
		EmaiOptions inOptions);

/*!
// @brief	Extracts friendly contact name from the contact list, at the specified index.
//
// @param	inContactList - contact list to be examined
// @param	inIndex - zero-based index of the contact in the list
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Zero-terminated unicode string which contains friendly contact name.
//				The function returns NULL if invalid parameters are specified
//				(i.e. inContactList is not valid list or inIndex is out of range)
//
// @see		Emai::ContactList::GetName
*/
EMAIAPI(EmaiConstUniCharPtr)
EmaiContactListGetContactName(
		EmaiContactList inContactList,
		EmaiUint32 inIndex,
		EmaiOptions inOptions);

/*!
// @brief	Extracts e-mail address from the contact list, at the specified index
//
// @param	inContactList - contact list to be examined
// @param	inIndex - zero-based index of the contact in the list
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Zero-terminated unicode string which contains contact's e-mail address.
//				The function returns NULL if invalid parameters are specified
//				(i.e. inContactList is not valid list or inIndex is out of range)
//
// @see		Emai::ContactList::GetAddress
*/
EMAIAPI(EmaiConstUniCharPtr)
EmaiContactListGetContactAddress(
		EmaiContactList inContactList,
		EmaiUint32 inIndex,
		EmaiOptions inOptions);


/*!
// @brief	Creates an emtpy content field object.
//
// @param	outContentField - on successful exit contains created object
// @param	inValue - corresponding field value:\n
//				For Content-Type field, should contain apropriate content type
//					(for example: "text/plain", "application/octet-stream", etc)
//				For Content-Disposition field, should contain apropriate disposition type
//					(for example: "attachment", "inline", etc)
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiContentFieldCreate(
		EmaiContentField* outContentField,
		EmaiConstUniCharPtr inValue,
		EmaiOptions inOptions);

/*!
// @brief	Adds parameter name and parameter value to the field object.
//
// @param	outContentField - on successful exit contains created object
// @param	inParamName - parameter name to be added
//							(for example: "filename")
// @param	inParamValue - parameter value to be added
//							(for example: "my file.txt")
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
// @see		Emai::ContentField::AddParameter
*/
EMAIAPI(EmaiError)
EmaiContentFieldAddParameter(
		EmaiContentField inContentField,
		EmaiConstUniCharPtr inParamName,
		EmaiConstUniCharPtr inParamValue,
		EmaiOptions inOptions);

/*!
// @brief	Extracts the value from the content field.
//
// @param	inContentField - content field to be examined
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Zero-terminated unicode string which contains value of the field.
//				The function returns NULL if invalid parameters are specified
//				(i.e. inContentField is not valid object)
//
// @see		Emai::ContentField::GetValue
*/
EMAIAPI(EmaiConstUniCharPtr)
EmaiContentFieldGetValue(
		EmaiContentField inContentField,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves the number of parameters at the content field.
//
// @param	inContentField - content field to be examined
// @param	outCount - on successful exit contains the number of the paramets 
//						at the content field
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
// @see		Emai::ContentField::GetParameterCount
*/
EMAIAPI(EmaiError)
EmaiContentFieldGetParameterCount(
		EmaiContentField inContentField,
		EmaiUint32* outCount,
		EmaiOptions inOptions);

/*!
// @brief	Extracts parameter name at the specified index, from the content field.
//
// @param	inContentField - content field to be examined
// @param	inIndex - zero-based index of the parameter in the field
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Zero-terminated unicode string which contains parameter name.
//				The function returns NULL if invalid parameters are specified
//				(i.e. inContentField is not valid object or inIndex is out of range)
//
// @see		Emai::ContentField::GetParameterName
*/
EMAIAPI(EmaiConstUniCharPtr)
EmaiContentFieldGetParameterName(
		EmaiContentField inContentField,
		EmaiUint32 inIndex,
		EmaiOptions inOptions);

/*!
// @brief	Extracts parameter value at the specified index, from the content field.
//
// @param	inContentField - content field to be examined
// @param	inIndex - zero-based index of the parameter in the field
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Zero-terminated unicode string which contains parameter value.
//				The function returns NULL if invalid parameters are specified
//				(i.e. inContentField is not valid object or inIndex is out of range)
//
// @see		Emai::ContentField::GetParameterValue
*/
EMAIAPI(EmaiConstUniCharPtr)
EmaiContentFieldGetParameterValue(
		EmaiContentField inContentField,
		EmaiUint32 inIndex,
		EmaiOptions inOptions);

/*!
// @brief	Finds parameter value with specified name, from the content field.
//
// @param	inContentField - content field to be examined
// @param	inParamName - parameter name to be found
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Zero-terminated unicode string which contains parameter value.
//				The function returns NULL if invalid parameters are specified
//				(i.e. inContentField is not valid object or parameter with the
//				specified name is not found in the field)
//
// @see		Emai::ContentField::FindParameterValue
*/
EMAIAPI(EmaiConstUniCharPtr)
EmaiContentFieldFindParameterValue(
		EmaiContentField inContentField,
		EmaiConstUniCharPtr inParamName,
		EmaiOptions inOptions);


/*!
// @brief	Creates message object from message part.
//
// @param	outMessage - on successful exit contains created message
// @param	inRootMessagePart - root message part, created using EmaiMessagePartCreateYYY
//								functions
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiMessageCreate(
		EmaiMessage* outMessage,
		EmaiMessagePart inRootMessagePart,
		EmaiOptions inOptions);

/*!
// @brief	Creates message object from MIME message source code
//
// @param	outMessage - on successful exit contains created message
// @param	inData - pointer to the RFC822 message source code
// @param	inDataSize - size (in bytes) of the message source code
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiMessageCreateFromData(
		EmaiMessage* outMessage,
		const void* inData,
		EmaiUint32 inDataSize,
		EmaiOptions inOptions);

/*!
// @brief	Options for EmaiMessageSetContactList, EmaiMessageCopyContactList,
//			EmaiMessageSetField and EmaiMessageGetFieldByName functions
//
// @see		EmaiMessageSetContactList
// @see		EmaiMessageCopyContactList
// @see		EmaiMessageSetField
// @see		EmaiMessageGetFieldByName
*/
enum EmaiEnumMessageFieldType
{
	EmaiFrom								= 1,
	EmaiTo,
	EmaiCC,
	EmaiSender,
	EmaiReplyTo,
	EmaiBCC,
	
	EmaiSubject,
	EmaiMimeVersion,
	EmaiXPriority,
};

/*!
// @brief	Adds/replaces field to/at the message header.
//
// @param	inMessage - message object to be modified
// @param	inFieldName - field name; if inFieldName is non-NULL, then 
//							inOptions parameter should be EmaiNullOptions;
/							if inFieldName is NULL, then  inOptions parameter should
//							specify one of the contact fields (EmaiFrom, EmaiTo, etc)
// @param	inFieldValue - field value, if NULL - the field is removed from the
//							message header
// @param	inOptions - EmaiNullOptions, or EmaiFrom, EmaiTo, etc
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
// @see		Emai::Message::SetField
*/
EMAIAPI(EmaiError)
EmaiMessageSetField(
		EmaiMessage inMessage,
		EmaiConstUniCharPtr inFieldName,		/*can be NULL*/
		EmaiConstUniCharPtr inFieldValue,		/*can be NULL*/
		EmaiOptions inOptions);


/*!
// @brief	Adds contact field to the message header.
//
// @param	inMessage - message object to be modified
// @param	inContactList - list of contacts
// @param	inOptions - specifies what kind of contact should be added
//							(EmaiFrom, EmaiTo, etc)
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiMessageSetContactList(
		EmaiMessage inMessage,
		EmaiContactList inContactList,
		EmaiOptions inOptions);


/*!
// @brief	Options for EmaiMessageCopyContactList, EmaiMessageGetFieldByName,
//			EmaiMessageGetFields functions
//			This option allows to control when corresponding functions
//			returns EmaiErrChartset error code. When this option is not
//			specified library ignores unknown charsets and returns
//			data as it appears in message source code.
//
// @see		EmaiMessageCopyContactList
// @see		EmaiMessageGetFieldByName
// @see		EmaiMessageGetFields
*/
enum
{
	EmaiFailOnUnknownCharset					= 0x8000000
};



/*!
// @brief	Encodings supported by the library
//
// @see		EmaiPartData
*/
enum EmaiEnumMessageEncoding
{
	EmaiEncodingAutomatic,
	EmaiEncodingBase64,
	EmaiEncodingQuotedPrintable,
	EmaiEncoding7bit,
	EmaiEncodingUnknown
};

/*
// The structures below are used to setup or retrieve data from the message part
*/

/*!
// @brief	plain/text
//
// @see		EmaiEnumMessageCharset
*/
typedef struct
{
	EmaiConstCharPtr			text;
	EmaiUint32					textSize;

	/*! kEmaiUSASCII etc*/
	EmaiUint32					characterSet;
} EmaiPlainTextPartData;

/*!
// @brief	plain/html
*/
typedef struct
{
	EmaiConstCharPtr			html;
	EmaiUint32					htmlSize;
} EmaiHtmlTextPartData;

/*!
// @brief	plain/text
*/
typedef struct
{
	EmaiConstUniCharPtr			text;

	/*! size of the text in EmaiUniChars*/
	EmaiUint32					textSize;
} EmaiUnicodeTextPartData;

/*!
// @brief	Attachment direct data
*/
typedef struct
{
	const void*					data;
	EmaiUint32					dataSize;
} EmaiDataPartData;

/*
// Attachment external file data
*/

/*!
// @brief	This structure allows to automatically load and encode external file
//			as attachment or inline image.
*/
typedef struct
{
	/*! zero terminating file path*/
	EmaiConstUniCharPtr			filePath;
} EmaiFilePartData;

/*!
// @brief	Composite part data
*/
typedef struct
{
	EmaiUint32					subPartCount;
	const EmaiMessagePart*		subParts;		/*!< can be NULL, in this case subPartCount should be 0*/
												/*! you can use EmaiMessagePartAddSubPart to add*/
												/*! subparts later*/
} EmaiCompositePartData;

/*!
// @brief	Values for partFormat member of EmaiPartData structure.\n
//			The following values are valid for EmaiStructureVersionFirst
//				version of EmaiPartData structure
//
// @see		EmaiPartData
*/
enum EmaiEnumMessagePartType
{
	EmaiPartPlainTextData,
	EmaiPartUnicodeTextData,
	EmaiPartHTMLData,
	EmaiPartDataData,
	EmaiPartFileData,
	EmaiPartCompositeData
};

/*!
// @brief	EmaiPartData
//
// @see		EmaiEnumMessageEncoding
// @see		EmaiEnumMessagePartType
*/
typedef struct
{
	/*! version of the structure*/
	/*! currently EmaiStructureVersionFirst*/
	EmaiUint32					version;

	/*
	// Members below are used to describe the most popular header fields
	*/

	/*! can be NULL if field is missing*/
	EmaiConstUniCharPtr			contentID;

	/*!can be NULL if field is missing*/
	EmaiContentField			contentType;

	/*! can be NULL if field is missing*/
	EmaiContentField			contentDisposition;

	/*! EmaiEncodingBase64, EmaiEncodingQuotedPrintable, etc, see @ref EmaiEnumMessageEncoding*/
	EmaiUint32					encoding;

	/*! selector for the union below, see @ref EmaiEnumMessagePartType*/
	EmaiUint32					partType;

	/*! This union allows you to specify or receive message part*/
	/*! data which is specific to the part type*/
	union
	{
		/*! Will be filled, when EmaiPartData::partType selector is set
			to EmaiEnumMessagePartType::EmaiPartPlainTextData value*/
		EmaiPlainTextPartData			plainText;

		/*! Will be filled, when EmaiPartData::partType selector is set
			to EmaiEnumMessagePartType::EmaiPartUnicodeTextData value*/
		EmaiUnicodeTextPartData			unicodeText;

		/*! Will be filled, when EmaiPartData::partType selector is set
			to EmaiEnumMessagePartType::EmaiPartHTMLData value*/
		EmaiHtmlTextPartData			htmlText;

		/*! Will be filled, when EmaiPartData::partType selector is set
			to EmaiEnumMessagePartType::EmaiPartDataData value*/
		EmaiDataPartData				data;

		/*! Will be filled, when EmaiPartData::partType selector is set
			to EmaiEnumMessagePartType::EmaiPartFileData value*/
		EmaiFilePartData				file;

		/*! Will be filled, when EmaiPartData::partType selector is set
			to EmaiEnumMessagePartType::EmaiPartCompositeData value*/
		EmaiCompositePartData			composite;
	}							partData;
	
} EmaiPartData;

/*!
// @brief	Creates message part from the specified data.
//
// @param	outMessagePart - on successful function return contains newly
//					created message part
// @param	inData - pointer to the data to be used for the part
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiMessagePartCreate(
		EmaiMessagePart* outMessagePart,
		const EmaiPartData* inData,
		EmaiOptions inOptions);

/*!
// @brief	Appends subparts to the composite part, created via EmaiMessagePartCreateComposite.
//
// @param	inCompositeMessagePart - composite message part to be modified
// @param	inSubParts - an array of message parts to be added to the composite part
// @param	inSubPartCount - number of items in inSubPart array
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
// @see		Emai::MessagePart::AddSubParts
*/
EMAIAPI(EmaiError)
EmaiMessagePartAddSubParts(
		EmaiMessagePart inCompositeMessagePart,
		const EmaiMessagePart inSubParts[],
		EmaiUint32 inSubPartCount,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves the root message part from message object .
//
// @param	inMessage - message object to be examined
// @param	outRootMessagePart - on successful exit contains root
//					message part object
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiMessageGetRootPart(
		EmaiMessage inMessage,
		EmaiMessagePart* outRootMessagePart,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves the message object from message part.
//
// @param	inMessagePart - message part object to be examined
// @param	outMessage - on successful exit contains message which contains the
//					message part, or NULL if the part wasn't added to the message
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiMessagePartGetMessage(
		EmaiMessagePart inMessagePart,
		EmaiMessage* outMessage,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves subparts from composite message part.
//
// @param	inCompositeMessagePart - message part object to be examined
// @param	outSubParts - if non-NULL on successful function exit this array
//					is filled with subparts, otherwise ioSubPartCount
//					will contain number of subparts
// @param	ioSubPartCount - if outSubParts is non-NULL, the number of items at the
//					outSubParts array is passed in function and number of actually
//					written items is returned on exit. If outSubParts is NULL, 
//					input value of the parameter is ignored, and number of
//					subparts is returned on function exit
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
// @see		Emai::MessagePart::GetSubParts
*/
EMAIAPI(EmaiError)
EmaiMessagePartGetSubParts(
		EmaiMessagePart inCompositeMessagePart,
		EmaiMessagePart outSubParts[],			/*!< can be NULL*/
		EmaiUint32* ioSubPartCount,
		EmaiOptions inOptions);

/*!
// @brief	Options for EmaiMessagePartGetData function
//
// @see		EmaiMessagePartGetData
*/
enum
{
	EmaiGetUnicodeTextData,
	EmaiGetNativeTextData
};

/*!
// @brief	Retrieves the data from the message part.
//
// @param	inMessagePart - message part object to be examined
// @param	outData - on successful function exit is filled with
//					message part data, this data is stored internally in the
//					message part object - you should not modify of release it
// @param	inOptions - EmaiGetUnicodeTextData or EmaiGetNativeTextData, this
//					parameter value is ignored when message part is not text
//					part, for text parts the parameter control how outData is
//					filled: as EmaiPartUnicodeTextData or as EmaiPartPlainTextData
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
// @see		Emai::MessagePart::GetData
*/
EMAIAPI(EmaiError)
EmaiMessagePartGetData(
		EmaiMessagePart inMessagePart,
		EmaiPartData* outData,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves fields from message header.
//
// @param	inMessage - message object to be examined
// @param	outFieldName - on exit will be filled with copy of field names,
//					caller is responsible for  releasing of returned list
// @param	outFieldValues - on exit will be filled with copy of field values,
//					caller is responsible for  releasing of returned list
// @param	inOptions - EmaiFailOnUnknownCharset or EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiMessageCopyFields(
		EmaiMessage inMessage,
		EmaiStringList* outFieldNames,
		EmaiStringList* outFieldValues,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves field content
//
// @param	inMessage - message object to be examined
// @param	inFieldName - field name which content should be retrieved
// @param	outFieldValue - pointer on EmaiConstUniCharPtr, which will be filled
//					with internaly stored decoded field content.
//					The value is undefined, if function returns
//					non-EmaiErrNoErr error code. The field value is stored
//					internally in message object and you should not modify or
//					release it
// @param	inOptions - EmaiFailOnUnknownCharset or EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
// @see		Emai::Message::GetField
*/
EMAIAPI(EmaiError)
EmaiMessageGetFieldByName(
		EmaiMessage inMessage,
		EmaiConstUniCharPtr inFieldName,
		EmaiConstUniCharPtr* outFieldValue,
		EmaiOptions inOptions);

/*
// @brief	Copies contact field from the message header.
//
// @param	inMessage - message object to be examined
// @param	outContactList - on exit will be filled with copy of the contact list
//					from the message, any modifications of the returned
//					contact list doesn't affect message contacts, caller is
//					responsible for freeing of the list using EmaiObjectRelease
// @param	inOptions - EmaiFrom, EmaiTo, etc. You can also combine this values
//					with EmaiFailOnUnknownCharset option
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
// @see		Emai::Message::GetContactList
// @see		EmaiObjectRelease
*/
EMAIAPI(EmaiError)
EmaiMessageCopyContactList(
		EmaiMessage inMessage,
		EmaiContactList* outContactList,
		EmaiOptions inOptions);


/*
// @brief	Retrieves message source.
//
// @param	inMessage - message object to be examined
// @param	outData - if non-NULL, on exit will be filled with address of the
//					begginning of the message source, this data is stored
//					internally in message object and you should not modify or
//					release it
// @param	outHeaderSize - if non-NULL, on exit will be filled with size of the
//					message header in bytes
// @param	outDataSize - if non-NULL, on exit will be filled with size of the
//					message source in bytes
// At least one of outData, outDataSize and outHeaderSize parameters should be
// non-NULL
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
// @see		Emai::Message::GetContactList
*/
EMAIAPI(EmaiError)
EmaiMessageGetData(
		EmaiMessage inMessage,
		const void** outData,		/*!< can be NULL*/
		EmaiUint32* outHeaderSize,	/*!< can be NULL*/
		EmaiUint32* outDataSize,	/*!< can be NULL*/
		EmaiOptions inOptions);


/*
// @brief	Retrieves message received date and time.
//
// @param	inMessage - message object to be examined
// @param	inTimeFormat - format printf-like string
// @param	outDateTime - on successful exit contains received formated date and time
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
//
*/
EMAIAPI(EmaiError)
EmaiMessageGetReceivedDateTime(
		EmaiMessage inMessage,
		tm* outDateTime,
		EmaiOptions inOptions);

#ifdef __cplusplus
	}
#endif

#endif /*__EMAIMESSAGE_H__*/
