/*!
// @file		EmaiPOP3.hpp
//
// @brief		C++ wrappers for POP3 functionality
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008-2010 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAIPOP3_HPP__
#define __EMAIPOP3_HPP__

#include <EmaiPOP3.h>
#include <EmaiMessage.hpp>

/*!
// @namespace	Emai
// @brief		Mailit for C/C++ library namespace
// @author		(C) 2006 Dacons LLP.
*/
namespace Emai
{

/*!
// @class Pop3StatusCallback
// @brief EmaiPop3StatusCallback wrapper class
//
// @see		Emai::Pop3Session::Pop3Session
*/
class Pop3StatusCallback
{
friend class Pop3Session;
	/*!
	// @brief	Virtual function
	//
	// @param	EmaiSmtpSession inSession - session object of the event
	// @param	EmaiUint32 inOperationCode - operation code
	// @param	EmaiConstCharPtr inStatusText - status text
	*/
	virtual EmaiError					OnOperation(
												Pop3Session& inSession,
												EmaiUint32 inOperationCode) = 0;

	/*!
	// @brief	Virtual function; will be triggered periodically while 
	//				sending the message, after a certain amount of data is received.
	//
	// @param	EmaiSmtpSession inSession - session object of the event
	// @param	EmaiUint32 inReceivedSize - data received amount, sinse last function call
	// @param	EmaiUint32 inTotalSize - total data amout to be sent
	*/
	virtual EmaiError					OnDataReceived(
												Pop3Session& inSession,
												EmaiUint32 inReceivedSize,
												EmaiUint32 inTotalSize) = 0;

	/*!
	// @brief	Virtual function; Will be triggered in case if server returns
	//				an error.
	//
	// @param	EmaiSmtpSession inSession - session object of the event
	// @param	EmaiConstCharPtr inServerReply - error reply string
	// @param	EmaiUint32 inReplyCode - error reply code
	*/
	virtual EmaiError					OnServerError(
												Pop3Session& inSession,
												EmaiConstCharPtr inServerReply,
												EmaiUint32 inReplyCode) = 0;
public:
	virtual								~Pop3StatusCallback();

protected:
	static EmaiError					StatusCallback(
												EmaiPop3Session inSession,
												const EmaiPop3CallbackData* inData,
												void* inUserData);
};

/*!
// @class MessageInfoList
// @brief EmaiPop3MessageInfoList wrapper class
*/
class MessageInfoList : public Object<EmaiPop3MessageInfoList>
{
protected:

	typedef Object<EmaiPop3MessageInfoList>	Parent;

										//! no implementation
										MessageInfoList();
										//! no implementation
										MessageInfoList(
												const MessageInfoList&);

public:
										
										MessageInfoList(
												EmaiConstUniCharPtr inFullFilePath,
												EmaiOptions inOptions = EmaiNullOptions);

										MessageInfoList(
												const void* inMessageListData,
												EmaiUint32 inMessageListDataSize,
												EmaiOptions inOptions = EmaiNullOptions);

										~MessageInfoList();

	void								EvaluateSublist(
												EmaiUint32 inDays,
												EmaiOptions inOptions = EmaiMessagesForDownload);

	/*!
	// @brief	Returns the number of information records, stored at the message info list.
	//
	// @param	inOptions - EmaiLocalMessages, EmaiRemoteMessages etc
	//
	// @return	Number of information records, stored at the message info list
	*/
	EmaiUint32							GetCount(
												EmaiOptions inOptions = EmaiMessagesForDownload);

	/*!
	// @brief	Retrives information record stored at the particular
	//			index of the message info list.
	//
	// @param	inMessageIndex - zero-based index of the record
	// @param	outInfo - pointer to the EmaiMessageInfo structure, which is
	//				filled with message information, on successful
	//				function exit
	// @param	inOptions - EmaiLocalMessages, EmaiRemoteMessages, etc.
	//
	// @see		EmaiMessageInfo
	*/
	void								GetInfo(
												EmaiUint32 inMessageIndex,
												EmaiMessageInfo& outInfo,
												EmaiOptions inOptions = EmaiMessagesForDownload);

	/*!
	// @brief	Marks the message at the specified index as "received", and saves it's info
	//			in the message history. This function will fail, if current sublist is
	//			not EmaiMessagesForDownload.
	//
	// @param	inMessageIndex - index of the message, actual interpretation of
	//					this parameter depends of the inOptions value, for
	//					more information see options value description
	// @param	inOptions - EmaiMessageIndexInList or EmaiMessageIndexOnServer
	*/
	void								MarkMessageReceived(
												EmaiUint32 inMessageIndex,
												EmaiOptions inOptions = EmaiMessageIndexInList);

	/*!
	// @brief	Retrives message info list data.
	//
	// @param	outData - pointer to message info list data
	// @param	outDataSize - message info list data size
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiMessageInfo
	*/
	void								GetMessageInfoListData(
												const void** outData,
												EmaiUint32& outDataSize,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Reset message info list.
	//
	// @param	inAccount - reset account string
	// @param	inServer - reset server string
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiMessageInfo
	*/
	void								ResetInfoList(
												EmaiConstUniCharPtr inAccount,
												EmaiConstUniCharPtr inServer,
												EmaiOptions inOptions = EmaiNullOptions);
};

/*!
// @class EmaiPop3Session
// @brief EmaiPop3Session wrapper class
*/
class Pop3Session : public Object<EmaiPop3Session>
{
protected:

	typedef Object<EmaiPop3Session>		Parent;

										//! no implementation
										Pop3Session();

public:
										
										Pop3Session(
												Pop3StatusCallback& inStatusCallback,
												EmaiUint32 inTimeout,
												EmaiOptions inOptions = EmaiNullOptions);
										Pop3Session(
												EmaiPop3StatusCallback inStatusCallback,
												void* inUserData,
												EmaiUint32 inTimeout,
												EmaiOptions inOptions = EmaiNullOptions);

										Pop3Session(
												const Pop3Session& inCopy);
										Pop3Session(
												EmaiPop3Session inSession,
												bool inAddRef);

										~Pop3Session();

	/*!
	// @brief	Establishes connection to the server.
	//
	// @param	inMailServer - pointer to null-teminated string which
	//						specifies name or IP address of POP3 server
	// @param	inPort - port number of POP3 server
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiEnumPop3DefaultPort
	// @see		EmaiPop3Connect()
	*/
	void								Connect(
												EmaiConstUniCharPtr inMailServer,
												short inPort = EmaiDefaultPop3Port,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Establishes connection to the server.
	//
	// @param	inMailServer - pointer to null-teminated string which
	//						specifies name or IP address of POP3 server
	// @param	inPort - port number of POP3 server
	// @param	inProxyInfo - proxy server information struct
	// @param	inUserData - user data specified parameter, use NULL
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiEnumPop3DefaultPort
	// @see		EmaiPop3ConnectEx()
	*/
	void								ConnectEx(
												EmaiConstUniCharPtr inMailServer,
												short inPort = EmaiDefaultPop3Port,
												const EmaiProxyInfo* inProxyInfo = NULL,
												void* inUserData = NULL,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Disconnects from the server.
	//
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiPop3Disconnect()
	*/
	void								Disconnect(
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Authenticates at the server.
	//
	// @param	EmaiConstCharPtr inAccount - pointer to null-teminated string, which conatins
	//								account name for authentication
	// @param	EmaiConstCharPtr inPassword - pointer to null-teminated string, which contains
	//								password for authentication
	// @param	EmaiOptions inOptions - preffered authentication method, i.e.
	//								EmaiPOP3AuthAutomatic, EmaiPOP3AuthLogin, etc.
	//
	// @see		EnumPop3AuthOptions
	// @see		EmaiPop3Authenticate()
	*/
	void								Authenticate(
												EmaiConstUniCharPtr inAccount,
												EmaiConstUniCharPtr inPassword,
												EmaiOptions inOptions = EmaiPOP3AuthAutomatic);

	/*!
	// @brief	Retrieves the list of messages, stored on the server.
	//
	// @param	inMessageInfoList - message list created using
	//				EmaiPop3CreateMessageInfoList function; initially this
	//				list contains messages which were received in previous
	//				pop3 sessions. It is up to user to pass correct list
	//				which contains message history for the e-mail account
	//				currenly user is working with.
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiPop3RetrieveMessageList()
	*/
	void								RetrieveMessageList(
												MessageInfoList& inMessageInfoList,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Retrieves the message from the server.
	//
	// @param	inMessageNumber - one-based message number on server
	// @param	inOptions - EmaiRetrieveRFC822Header for retrieving message header only, 
	//				EmaiNullOptions retrives full message
	//
	// @return	Message object
	//
	// @see		EmaiPop3RetrieveMessage()
	*/
	Message								RetrieveMessage(
												EmaiUint32 inMessageNumber,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Marks message at specified index on server as "deleted".
	//
	// @param	inMessageNumber - message number on the server
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiPop3DeleteMessage()
	*/
	void								DeleteMessage(
												EmaiUint32 inMessageNumber,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Unmarks all messages as "deleted" on the server.
	//
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiPop3Reset()
	*/
	void								Reset(
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Marks POP3 session to be cancelled ASAP
	//
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiPop3Cancel()
	*/
	void								Cancel(
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Sens NOOP command to the POP3 server.
	//			Used to avoid timout disconnection, during inactivity period.
	//
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiPop3Noop()
	*/
	void								Noop(
												EmaiOptions inOptions = EmaiNullOptions);
};

/*
// Pop3StatusCallback class implemetation
*/
inline EmaiError
Pop3StatusCallback::StatusCallback(
		EmaiPop3Session inSession,
		const EmaiPop3CallbackData* inData,
		void* inUserData
	)
{
	Pop3StatusCallback* callback = reinterpret_cast<Pop3StatusCallback*>(inUserData);
	EmaiError error = EmaiErrNoErr;
	try
	{
		if (callback != NULL)
		{
			Pop3Session session(inSession, true);
			switch (inData->selector)
			{
				case EmaiPop3SelectorNormal:
					error = callback->OnOperation(session, inData->data.normal.operationCode);
					break;
				case EmaiPop3SelectorReceiving:
					error = callback->OnDataReceived(session, inData->data.receiving.receivedSize,
										inData->data.receiving.totalSize);
					break;
				case EmaiPop3SelectorServerError:
					error = callback->OnServerError(session, inData->data.serverError.serverReply,
										inData->data.serverError.replyCode);
					break;
				default:
					error = EmaiErrUnknown;
					break;
			}
		}
	}
	catch (...)
	{
		//do not pass exceptions in library
		error = EmaiErrUnknown;
	}
	return error;
}

inline
Pop3StatusCallback::~Pop3StatusCallback()
{
}

/*
// MessageInfoList class implemetation
*/
inline
MessageInfoList::MessageInfoList(
		EmaiConstUniCharPtr inFullFilePath,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3MessageInfoListCreateFromFile(&mObject,
					inFullFilePath, inOptions));
}

inline
MessageInfoList::MessageInfoList(
		const void* inMessageListData,
		EmaiUint32 inMessageListDataSize,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3MessageInfoListCreateFromData(&mObject,
					inMessageListData, inMessageListDataSize, inOptions));
}

inline
MessageInfoList::~MessageInfoList()
{
}

inline void
MessageInfoList::EvaluateSublist(
		EmaiUint32 inDays,
		EmaiOptions inOptions /*= EmaiMessagesForDownload*/
	)
{
	CheckErrorCode(EmaiPop3MessageInfoListEvaluateSubList(mObject,
					inDays, inOptions));
}

inline EmaiUint32
MessageInfoList::GetCount(
		EmaiOptions inOptions /*= EmaiMessagesForDownload*/
	)
{
	EmaiUint32 count = 0;
	CheckErrorCode(EmaiPop3MessageInfoListGetCount(mObject,
					&count, inOptions));
	return count;
}

inline void
MessageInfoList::GetInfo(
		EmaiUint32 inMessageIndex,
		EmaiMessageInfo& outInfo,
		EmaiOptions inOptions /*= EmaiMessagesForDownload*/
	)
{
	CheckErrorCode(EmaiPop3MessageInfoListGetInfo(mObject,
					inMessageIndex, &outInfo, inOptions));
}

inline void
MessageInfoList::MarkMessageReceived(
		EmaiUint32 inMessageIndex,
		EmaiOptions inOptions /*= EmaiMessageIndexInList*/
	)
{
	CheckErrorCode(EmaiPop3MessageInfoMarkMessageReceived(mObject,
					inMessageIndex, inOptions));
}

inline void
MessageInfoList::GetMessageInfoListData(
		const void** outData,
		EmaiUint32& outDataSize,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3MessageInfoListGetHistoryData(mObject,
					outData, outDataSize, inOptions));
}

inline void
MessageInfoList::ResetInfoList(
		EmaiConstUniCharPtr inAccount,
		EmaiConstUniCharPtr inServer,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3MessageInfoListResetHistory(mObject,
					inAccount, inServer, inOptions));
}
/*
// Pop3Session class implemetation
*/
inline
Pop3Session::Pop3Session(
		Pop3StatusCallback& inStatusCallback,
		EmaiUint32 inTimeout,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3SessionCreate(&mObject,
					&Pop3StatusCallback::StatusCallback, &inStatusCallback,
					inTimeout, inOptions));
}

inline
Pop3Session::Pop3Session(
		EmaiPop3StatusCallback inStatusCallback,
		void* inUserData,
		EmaiUint32 inTimeout,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3SessionCreate(&mObject,
					inStatusCallback, inUserData,
					inTimeout, inOptions));
}

inline
Pop3Session::Pop3Session(
		const Pop3Session& inCopy
	):
	Parent(inCopy)
{
}

inline
Pop3Session::Pop3Session(
		EmaiPop3Session inSession,
		bool inAddRef
	):
	Parent(inSession, inAddRef)
{
}

inline
Pop3Session::~Pop3Session()
{
}

inline void
Pop3Session::Connect(
		EmaiConstUniCharPtr inMailServer,
		short inPort /*= EmaiDefaultPop3Port*/,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3Connect(mObject, inMailServer, inPort, inOptions));
}

inline void
Pop3Session::ConnectEx(
		EmaiConstUniCharPtr inMailServer,
		short inPort /*= EmaiDefaultPop3Port*/,
		const EmaiProxyInfo* inProxyInfo /*= NULL*/,
		void* inUserData,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3ConnectEx(mObject, inMailServer, inPort, inProxyInfo, inUserData, inOptions));
}

inline void
Pop3Session::Disconnect(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3Disconnect(mObject, inOptions));
}

inline void
Pop3Session::Authenticate(
		EmaiConstUniCharPtr inAccount,
		EmaiConstUniCharPtr inPassword,
		EmaiOptions inOptions /*= EmaiPOP3AuthAutomatic*/
	)
{
	CheckErrorCode(EmaiPop3Authenticate(mObject, inAccount,
							inPassword, inOptions));
}

inline void
Pop3Session::RetrieveMessageList(
		MessageInfoList& inMessageInfoList,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3RetrieveMessageList(mObject, inMessageInfoList,
							inOptions));
}

inline Message
Pop3Session::RetrieveMessage(
		EmaiUint32 inMessageNumber,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiMessage message = NULL;
	CheckErrorCode(EmaiPop3RetrieveMessage(mObject, &message,
							inMessageNumber, inOptions));
	return Message(message, false);
}

inline void
Pop3Session::DeleteMessage(
		EmaiUint32 inMessageNumber,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3DeleteMessage(mObject, inMessageNumber, inOptions));
}

inline void
Pop3Session::Reset(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3Reset(mObject, inOptions));
}

inline void
Pop3Session::Cancel(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3Cancel(mObject, inOptions));
}

inline void
Pop3Session::Noop(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiPop3Noop(mObject, inOptions));
}


}


#endif /*__EMAIPOP3_HPP__*/
